#!/usr/bin/python3
"""
// © University of Southampton IT Innovation Centre, 2018
//
// Copyright in this software belongs to University of Southampton
// IT Innovation Centre of Gamma House, Enterprise Road,
// Chilworth Science Park, Southampton, SO16 7NS, UK.
//
// This software may not be used, sold, licensed, transferred, copied
// or reproduced in whole or in part in any manner or form or in or
// on any media by any person other than in accordance with the terms
// of the Licence Agreement supplied with the software, or otherwise
// without the prior written consent of the copyright owners.
//
// This software is distributed WITHOUT ANY WARRANTY, without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE, except where stated in the Licence Agreement supplied with
// the software.
//
//      Created By :            Nikolay Stanchev
//      Created Date :          15-05-2018
//      Created for Project :   FLAME
"""

from os import path
from json import load
from pyramid.config import Configurator
from sqlalchemy import engine_from_config
from clmcservice.models.meta import DBSession, Base
from clmcservice.aggregationapi.utilities import validate_conf_file, RUNNING_FLAG, MALFORMED_FLAG, CONF_FILE_ATTRIBUTE, CONF_OBJECT, AGGREGATOR_CONFIG_SECTION


def main(global_config, **settings):
    """
    This function returns a Pyramid WSGI application.
    """

    engine = engine_from_config(settings, 'sqlalchemy.')  # initialise a database engine by using the 'sqlalchemy' setting in the configuration .ini file
    DBSession.configure(bind=engine)  # bind the engine to a DB session
    Base.metadata.bind = engine  # bind the engine to the Base class metadata

    # validate and use (if valid) the configuration file
    conf_file_path = settings[CONF_FILE_ATTRIBUTE]
    conf = validate_conf_file(conf_file_path)  # if None returned here, service is in unconfigured state
    settings[CONF_OBJECT] = conf

    settings[MALFORMED_FLAG] = False
    settings['influx_port'] = int(settings['influx_port'])  # the influx port setting must be converted to integer instead of a string

    network_config_file_path = settings["network_configuration_path"]
    with open(network_config_file_path) as f:
        network = load(f)
        settings["network_bandwidth"] = network["bandwidth"]

    config = Configurator(settings=settings)

    # add routes of the aggregator API
    config.add_route('aggregator_config', '/aggregator/config')
    config.add_route('aggregator_controller', '/aggregator/control')
    config.add_route('round_trip_time_query', '/query/round-trip-time')

    # add routes of the WHOAMI API
    config.add_route('whoami_endpoints', '/whoami/endpoints')
    config.add_route('whoami_endpoints_instance', 'whoami/endpoints/instance')

    # add routes of the CONFIG API
    config.add_route('config_sfc', '/config/sf-chains')
    config.add_route('config_sfc_instance', '/config/sf-chains/instance')

    # add routes of the GRAPH API
    config.add_route('graph_build', '/graph/temporal')
    config.add_route('graph_manage', '/graph/temporal/{graph_id}')
    config.add_route('graph_algorithms_rtt', '/graph/temporal/{graph_id}/round-trip-time')

    config.scan()  # This method scans the packages and finds any views related to the routes added in the app configuration
    return config.make_wsgi_app()
